"""This database module is used to work with the Message table."""

# Created: Sat Oct 28 23:48:13 PDT 2000
# Author: Shannon -jj Behrens
# Email: jjinux@users.sourceforge.net
#
# Copyright (c) Shannon -jj Behrens.  All rights reserved.

from aquarium.util.AquariumClass import AquariumClass


class Message(AquariumClass):

    """This database class is used to work with the Message table."""

    def insert(self, subject="", name="", email="", parent=0, body=""):
        """Insert a new message"""
        esc = self._ctx.db.escape_string
        cursor = self._ctx.dba.cursor()
        subject = esc(subject)
        name = esc(name)
        email = esc(email)
        parent = abs(int(parent))
        body = esc(body)
        query = """\
            insert into Message (id, subject, name, email,
                created, parent, body)
            values (0, '%(subject)s', '%(name)s', '%(email)s', now(),
                    %(parent)s, '%(body)s')
        """ % vars()
        cursor.execute(query)

    def get(self, id, force=1):
        """Return a particular Message as a dict given it's id.

        The force parameter works as it does in the fetchonedict method of
        DatabaseAssistant.

        """
        cursor = self._ctx.dba.cursor()
        query = "select * from Message where id = %s" % abs(int(id))
        cursor.execute(query)
        return self._ctx.dba.fetchonedict(cursor, force)

    def list(self):
        """Get a skeleton of all of the messages.

        I'll return a list of dicts.  By skeleton, I mean that I will
        not be returning the body field.  I'll order by parent and
        then created.

        """
        cursor = self._ctx.dba.cursor()
        query = """\
            select id, subject, name, email, created, parent
            from Message
            order by parent, created
        """
        cursor.execute(query)
        return self._ctx.dba.fetchalldicts(cursor)
